import os
from os.path import join, splitext, basename
from glob import glob
from typing import Set

from osgeo import gdal

from ..util import replace_ext


def extract_subdataset_step(source_dir: os.PathLike, target_dir: os.PathLike, data_file_glob: str, subdataset_types: Set[str]=None):
    for filename in glob(join(source_dir, data_file_glob)):
        extract_subdatasets(
            filename,
            join(target_dir, basename(replace_ext(filename, '.tif'))),
            subdataset_types
        )


def extract_subdatasets(source_filename: os.PathLike, target_filename: os.PathLike, subdataset_types: Set[str]=None):
    ds = gdal.Open(source_filename)

    sub_datasets = []
    for locator, _ in ds.GetSubDatasets():
        _, _, sd_type = locator.split(':')
        if subdataset_types is None or sd_type in subdataset_types:
            sub_datasets.append(locator)

    # combine as VRT
    out_vrt_name = join(source_dir, replace_ext(filename, '.vrt'))
    gdal.BuildVRT(out_vrt_name, sub_datasets, # TODO: options
    )

    # TODO: maybe just translate here?
    gdal.Warp(
        target_filename,
        out_vrt_name
    )
